/*****
  file: serial_example.c
  Description: RS232 interface test
  date: September 28, 2001
  author: Emil Jovanov
***/  
  
#include <msp430x14x.h>
#include <string.h>
#include "rs232.h"

//#define USING_8MHZ_CLOCK

#define LED_SETUP()   P1DIR |= BIT0 + BIT1     // P1.0 & P1.1 output to drive LEDs

#define LEDON0     P1OUT |= BIT0
#define LEDOFF0    P1OUT &= ~BIT0
#define LEDTOGGLE0 P1OUT ^= ~BIT0

#define LEDON1     P1OUT |= BIT1
#define LEDOFF1    P1OUT &= ~BIT1


#ifndef _RS232_CONFIG_H_
#define _RS232_CONFIG_H_

#define USE_UART0  // uncomment this to compile in functions for UART0
//#define USE_UART1  // uncomment this to compile in functions for UART1

/* uncomment this to make UART interrupt driven */
//#define UART0_INT_DRIVEN
//#define UART1_INT_DRIVEN

#define UART0_RX_BUFSIZE  64
#define UART0_TX_BUFSIZE  64

#endif //_RS232_CONFIG_H_


//Delay some time
void delay(void)
{
  uint16 i;
  for (i = 65000; i > 0; i--);
  for (i = 65000; i > 0; i--);
}

////////////////////////////////////////////////////////////////
//   UART0 variables
////////////////////////////////////////////////////////////////
#ifdef UART0_INT_DRIVEN
#ifdef USE_UART0
char uart0_rx_buf[UART0_RX_BUFSIZE];
char uart0_tx_buf[UART0_TX_BUFSIZE];

uint8 uart0_rx_head, uart0_rx_tail;
uint8 uart0_tx_head, uart0_tx_tail;
uint8 uart0_flags;

#endif // USE_UART0
#endif // UART0_INT_DRIVEN

////////////////////////////////////////////////////////////////
//   UART1 variables
////////////////////////////////////////////////////////////////
#ifdef UART1_INT_DRIVEN
#ifdef USE_UART1
char uart1_rx_buf[UART1_RX_BUFSIZE];
char uart1_tx_buf[UART1_TX_BUFSIZE];

uint8 uart1_rx_head, uart1_rx_tail;
uint8 uart1_tx_head, uart1_tx_tail;
uint8 uart1_flags;

#endif //USE_UART1
#endif // UART1_INT_DRIVEN


////////////////////////////////////////////////////////////////
//   UART0 functions
////////////////////////////////////////////////////////////////
#ifdef USE_UART0
#ifdef UART0_INT_DRIVEN

void UART0_ClearTxBuffer(void)
{
  uart0_tx_head = 0;
  uart0_tx_tail = 0;
}

void UART0_ClearRxBuffer(void)
{
  uart0_rx_head = 0;
  uart0_rx_tail = 0;
}

bool UART0_TxBufferEmpty(void)
{
  return (uart0_tx_head == uart0_tx_tail);
}

bool UART0_RxBufferEmpty(void)
{
  return (uart0_rx_head == uart0_rx_tail);
}

bool UART0_TxBufferFull(void)
{
  uint8 next = uart0_tx_head + 1;
  if (next >= UART0_TX_BUFSIZE)
    next = 0;

  return (next == uart0_tx_tail);
}

bool UART0_RxBufferFull(void)
{
  uint8 next = uart0_rx_head + 1;
  if (next >= UART0_RX_BUFSIZE)
    next = 0;

  return (next == uart0_rx_tail);
}
#endif //UART0_INT_DRIVEN


bool UART0_RXready(void)
{
#ifdef UART0_INT_DRIVEN
  return (uart0_rx_head != uart0_rx_tail);
#else
  return ((IFG1 & URXIFG0) != 0);
#endif
}

char UART0_getchar(void)
{
#ifdef UART0_INT_DRIVEN
  char c;

  /* wait forever for character to arrive */
  while (uart0_rx_tail == uart0_rx_head);

  c = uart0_rx_buf[uart0_rx_head];
  if (++uart0_rx_head >= UART0_RX_BUFSIZE)
    uart0_rx_head = 0;
  
  return c;
#else
  for (;;)
  {
    if (U0RCTL & RXERR)
    {
      // if you want to answer receive errors, do it here.
      U0RCTL &= ~(FE+PE+OE+BRK+RXERR);
    }
    else
    {
      if (IFG1 & URXIFG0)
      {  // we've received a character
        return U0RXBUF;
      }
    }
  }
#endif //UART0_INT_DRIVEN
}

char UART0_putchar(char c)
{
#ifdef UART0_INT_DRIVEN
  uint8 next;

  next = uart0_tx_head + 1;
  if (next >= UART0_TX_BUFSIZE)
    next = 0;

  /* wait forever until buffer is not full */
  /* apr 25 01 - jcc changed to dump bytes instead of waiting */
  //while (next == uart0_tx_tail);
  if (next == uart0_tx_tail)
     return 0;
  uart0_tx_buf[uart0_tx_head] = c;  // store character in buffer
  uart0_tx_head = next;

  // if not already transmitting, then start transmit interrupt
  if (!(uart0_flags & UART_TRANSMITTING))
  {
    U0TXBUF = uart0_tx_buf[uart0_tx_tail];
    if (++uart0_tx_tail >= UART0_TX_BUFSIZE)
      uart0_tx_tail = 0;
    uart0_flags |= UART_TRANSMITTING;      /* set transmitting flag */
  }
  
  return c;
#else
  // wait for other character to transmit
  while (!(IFG1 & UTXIFG0));
  
  U0TXBUF = c;
  return c;
#endif //UART0_INT_DRIVEN
}

void UART0_putstring(char *s)
{
  while (*s) UART0_putchar(*s++);
}

#endif // USE_UART0


////////////////////////////////////////////////////////////////
//   UART1 functions
////////////////////////////////////////////////////////////////
#ifdef USE_UART1
#ifdef UART1_INT_DRIVEN
void UART1_ClearTxBuffer(void)
{
  uart1_tx_head = 0;
  uart1_tx_tail = 0;
}

void UART1_ClearRxBuffer(void)
{
  uart1_rx_head = 0;
  uart1_rx_tail = 0;
}

bool UART1_TxBufferEmpty(void)
{
  return (uart1_tx_head == uart1_tx_tail);
}

bool UART1_RxBufferEmpty(void)
{
  return (uart1_rx_head == uart1_rx_tail);
}

bool UART1_TxBufferFull(void)
{
  uint8 next = uart1_tx_head + 1;
  if (next >= UART1_TX_BUFSIZE)
    next = 0;

  return (next == uart1_tx_tail);
}

bool UART1_RxBufferFull(void)
{
  uint8 next = uart1_rx_head + 1;
  if (next >= UART1_RX_BUFSIZE)
    next = 0;

  return (next == uart1_rx_tail);
}
#endif //UART1_INT_DRIVEN


bool UART1_RXready(void)
{
#ifdef UART1_INT_DRIVEN
  return (uart1_rx_head != uart1_rx_tail);
#else
  return ((IFG2 & URXIFG1) != 0);
#endif
}

char UART1_getchar(void)
{
#ifdef UART1_INT_DRIVEN
  char c;

  /* wait forever for character to arrive */
  while (uart1_rx_tail == uart1_rx_head);

  c = uart1_rx_buf[uart1_rx_head];
  if (++uart1_rx_head >= UART1_RX_BUFSIZE)
    uart1_rx_head = 0;
  
  return c;
#else
  for (;;)
  {
    if (U1RCTL & RXERR)
    {
      // if you want to answer receive errors, do it here.
      U1RCTL &= ~(FE+PE+OE+BRK+RXERR);
    }
    else
    {
      if (IFG2 & URXIFG1)
      {  // we've received a character
        return U1RXBUF;
      }
    }
  }
#endif //UART1_INT_DRIVEN
}


char UART1_putchar(char c)
{
#ifdef UART1_INT_DRIVEN
  uint8 next;

  next = uart1_tx_head + 1;
  if (next >= UART1_TX_BUFSIZE)
    next = 0;

  /* wait forever until buffer is not full */
  while (next == uart1_tx_tail);



  uart1_tx_buf[uart1_tx_head] = c;  // store character in buffer
  uart1_tx_head = next;
  
  // if not already transmitting, then start transmit interrupt
  if (!(uart1_flags & UART_TRANSMITTING))
  {
    U1TXBUF = uart1_tx_buf[uart1_tx_tail];
    if (++uart1_tx_tail >= UART1_TX_BUFSIZE)
      uart1_tx_tail = 0;
    uart1_flags |= UART_TRANSMITTING;      /* set transmitting flag */
  }
  
  return c;
#else
  // wait for other character to transmit
  while (!(IFG2 & UTXIFG1));
  
  U1TXBUF = c;
  return c;
#endif //UART1_INT_DRIVEN
}

void UART1_putstring(char *s)
{
  while (*s) UART1_putchar(*s++);
}
#endif // USE_UART1


void UART_Initialize(void)
{
////////////////////////////////////////////////////////////////
//   UART0 Initialization
////////////////////////////////////////////////////////////////
#ifdef USE_UART0

// UART0 - 38400 bps
  U0MCTL = 0x10;
  U0BR0  = 0x68;
  U0BR1  = 0x00;

  P3SEL |= 0x30; // bits 4-5 are for special function UART0
  ME1 |= URXE0 + UTXE0; // UART module enable
  U0CTL = CHAR;     /* 8-bit characters */
  U0TCTL = SSEL0; /* clock source ACLK */
  U0RCTL = 0;

#ifdef UART0_INT_DRIVEN
  uart0_rx_head = 0;
  uart0_rx_tail = 0;
  uart0_tx_head = 0;
  uart0_tx_tail = 0;
  uart0_flags = 0;
  IE1 |= UTXIE0 + URXIE0;  // enable TX and RX interrupts
#endif // UART0_INT_DRIVEN
#endif // USE_UART0

////////////////////////////////////////////////////////////////
//   MAXIM CHIP Initialization
////////////////////////////////////////////////////////////////
#ifdef USE_MAXIM_CE
  P1DIR |= MAXIM_CE_PIN;
  P1SEL &= ~MAXIM_CE_PIN;
  //reverse logic to enable maxim chip; set CE to low
  P1OUT &= ~MAXIM_CE_PIN;
#endif //USE_MAXIM_CE
}  //end UART_Initialize()

#ifdef UART0_INT_DRIVEN
#ifdef USE_UART0

/* RS232 receive interrupt service routine */
interrupt [UART0RX_VECTOR] void UART0_RX_interrupt(void)
{
  uint8 next;

  /* if we received a bad character, just clear flags and discard. */
  if (U0RCTL & RXERR)
    U0RCTL &= ~(FE+PE+OE+BRK+RXERR);
//  else
  {
    next = uart0_rx_tail + 1;
    if (next >= UART0_RX_BUFSIZE)
      next = 0;

    if (next != uart0_rx_head)
    {
      uart0_rx_buf[uart0_rx_tail] = U0RXBUF;
      uart0_rx_tail = next;
      uart0_flags &= ~UART_RX_OVERFLOW;     /* clear RX buffer full flag */
    }
    else
    {
      uart0_flags |= UART_RX_OVERFLOW;      /* set RX buffer full flag */
    }
  }
}


/* RS232 transmit interrupt service routine */
interrupt [UART0TX_VECTOR] void UART0_TX_interrupt(void)
{
  if (uart0_tx_tail != uart0_tx_head) /* check if buffer is empty */
  {

    U0TXBUF = uart0_tx_buf[uart0_tx_tail];
    if (++uart0_tx_tail >= UART0_TX_BUFSIZE)
      uart0_tx_tail = 0;
    uart0_flags |= UART_TRANSMITTING;       /* set transmitting flag */

  }
  else
  {
    uart0_flags &= ~UART_TRANSMITTING;      /* clear transmitting flag */
  }
}

#endif //USE_UART0
#endif // UART0_INT_DRIVEN

void SpeedUpClock(void)
{
  IFG2 = 0;
  IFG1 = 0;

#ifdef USING_8MHZ_CLOCK
  BCSCTL1 |= XTS+DIVM0;
#else  //using a 4MHz clock
  BCSCTL1 |= XTS;
#endif  //USING_8_MHZ_CLOCK

  /*wait in loop until crystal stabalizes*/
  do
  {
    IFG1 &= ~OFIFG;
  }
  while (OFIFG & IFG1);

  IFG1 &= ~OFIFG;

  /*Reset osc. fault flag again
  *Then set MCLK same as LFXT1CLK
  *and SMCLK = XT2CLK
  */
  BCSCTL2 = SELM0 + SELM1;
}

void main(void)
{

  unsigned int ii;
  
  WDTCTL = WDTPW + WDTHOLD; // Stop watchdog timer

  P4DIR = BIT0 + BIT2 + BIT3; // P40,2,3 for LINX wireless device
  P1DIR = BIT0 + BIT1;     // P1.0 & P1.1 output to drive LEDs
  P1OUT=0;
  P4OUT=0;

  SpeedUpClock();

  delay();
  LEDON0;
  delay();
  LEDOFF0;

  UART_Initialize();

  _EINT();       // enable global interrupts

  // send a message
    UART0_putchar('C');
    UART0_putchar('P');
    UART0_putchar('E');
    UART0_putchar('6');
    UART0_putchar('2');
    UART0_putchar('1');

  while(1) 
    {
    delay();
    UART0_putchar('*');		// send something
    }

}

